const { Group, Field, randomId, parseArgs } = require('@igomoon/hubspot-fields-js');
const path = require('path');
const { logger } = require('@hubspot/cli-lib/logger')
/**
 * Load Field and Clear Cache from fields folder in src
 */
let globalFieldsDirPath = null
const setGlobalFieldsDirPath = (path) => {
	globalFieldsDirPath = path
	return globalFieldsDirPath
}

let usePackageFields = true
const setEnablePackageFields = (val) => {
	usePackageFields = !!val
	return usePackageFields
}

const loadFromFieldsDir = (module, lookInPackage = true) => {

	if (module.startsWith('#/') && lookInPackage) {
		return loadFromPackageFieldsDir(module.substr(2))
	}

	if (!globalFieldsDirPath) {
		if (usePackageFields && lookInPackage) {
			return loadFromPackageFieldsDir(module)
		}
		throw new Error('Global Fields Directory Path not set. Use setGlobalFieldsDirPath(path).')
	}

	try {
		let pathToField = path.resolve(globalFieldsDirPath, module);
		delete require.cache[require.resolve(pathToField)];
		return require(pathToField);
	} catch (e) {
		if (usePackageFields && lookInPackage) {
			return loadFromPackageFieldsDir(module)
		}
		logger.error(`Could not find file "${module}" in assigned fields dir: ${globalFieldsDirPath}`, e.requireStack || '')
		throw e
    }
}

const loadFromPackageFieldsDir = (module) => {
	if (!usePackageFields) {
		throw new Error('Use fields from the package directory is disabled. Use setEnablePackageFields(true) to enable.')
	}
	try {
		let pathToField = path.resolve(__dirname + '/lib', module);
		delete require.cache[require.resolve(pathToField)];
		return require(pathToField);
	} catch (e) {
		logger.error("Could not find file in package fields dir: " + module, e.requireStack)
		throw e
    }
}

/**
 * ID Helper
 */
let fieldJsGeneratedIds = {}
const getOrCreateId = (key) => {
	if (!fieldJsGeneratedIds[key]) { fieldJsGeneratedIds[key] = randomId()}
	return fieldJsGeneratedIds[key];
}

/**
 * Exports
 */
module.exports = {
	// Fields Dir
	setGlobalFieldsDirPath,
	setEnablePackageFields,
	loadFromFieldsDir,
	loadFromPackageFieldsDir,
	// Id Helper
	getOrCreateId,
	// Field JS Package
	Group,
	Field,
	randomId,
	parseArgs
};