const { Group, Field, randomId } = require('../../../build');
let GlobalGroup = require('../includes/GlobalGroup');

let Heading = require('../Heading');
let BasicText = require('../BasicText');
let Columns = require('../Columns');
let Colors = require('../Colors');
let Image = require('./Image');
let Icon = require('./Icon');
let Button = require('./Button');
let CssClass = require('../CssClass');

module.exports = class extends GlobalGroup {

	constructor() { 
		super();
		this.name("blurbs", "Blurbs")
		this.id(randomId())
		this.mediaTypeId = randomId()
		this.linkTypeId = randomId()
		this._build()
		return this
	}

	_build() {
		return this._buildChildrenArray(() => {
			this.appendChildren(this.#standardFields());
		})
	}

	// --- Fields

	#standardFields() {
		return [
			new Group()
                .name('items', 'Blurbs')
                .children([
                    new Group()
                        .name('text', 'Text')
                        .children([
                            new Heading()
                                .inlineHelpText('This is the Heading for the Blurb')
                                .default('I\'m the Blurb Heading'),
                            new BasicText()
                                .name('description', 'Description')
                                .default('Quis aute iure reprehenderit in voluptate velit esse. Quid securi etiam tamquam eu fugiat nulla pariatur. Excepteur sint obcaecat cupiditat non proident culpa.')
                        ]),
                    new Icon()
                        .visibleIf(this.mediaTypeId, 'icon'),
                    new Image()
						.visibleIf(this.mediaTypeId, 'image')
						.showRatio()
						.showSize(),
                    new Group()
						.name('animated_number', 'Animated numbers')
						.visibleIf(this.mediaTypeId, 'animation')
                        .children([
                            Field.number()
                                .name('number', 'Animated number')
                                .inlineHelpText('This number will be animated')
                                .default(2),
                            Field.text()
                                .name('prefix', 'Prefix')
                                .inlineHelpText('If you want to add a prefix to the number, e.g. $, €'),
                            Field.text()
                                .name('suffix', 'Suffix')
                                .inlineHelpText('If you want to add a suffix to the number, e.g. %, +, k'),                         
                            Field.text()
                                .name('subtitle', 'Sub title')
                                .inlineHelpText('Shown under the number, e.g. Customers')                        
						]),
					
					new Button()
						.name('buttons', 'Buttons')
						.visibleIf(this.linkTypeId, 'buttons')
						.default([])
						.repeat({
							"min": 0,
							"max": 2,
							"sorting_label_field": this.data.id + '.items.buttons.button_label',
							"default": 0
						}),
					
					Field
						.link()
						.name('blurb_link', 'Blurb Link')
                        .visibleIf(this.linkTypeId, 'anywhere'),
					
					Field
						.boolean()
						.name('visible', 'Enable this Blurb')
						.set('display', 'toggle')
						.inlineHelpText("Check to show blurb.")
						.default(true),
					
					new CssClass()
					
                ]).repeat({
                    'min' : 0,
                    'max' : null,
                    'sorting_label_field' : this.data.id + '.items.text.heading',
                    'default' : 3
				}),
			
            new Group()
                .name('layout', 'Blurbs Style')
				.children([
					Field
						.choice()
						.id(this.mediaTypeId)
						.choices([
							['none', 'None'],
							['icon', 'Icons'],
							['image', 'Large Images'],
							['animation', 'Animated numbers']
						])
						.name('media_type', 'Visual aid above blurb text')
						.default('none'),	
					Field
						.choice()
						.id(this.linkTypeId)
						.choices([
							['buttons', 'Buttons'],
							['anywhere', 'Make whole blurb clickable'],
						])
						.name('link_type', 'Clickable Type')
						.default('buttons'),
					new Columns(),
                    new Colors()
                        .name('blurb_bg', 'Background color')
                        .inlineHelpText('Adds a background to each blurb for a card look'),
					new Colors()
						.name('border_color', 'Border color'),
					Field
						.boolean()
						.name('inline', 'Inline Layout')
						.set('display', 'toggle')
						.visibleIf(this.mediaTypeId, 'icon')
						.default(false),
                    Field
                        .boolean()
                        .name('equal_height', 'Make blurbs equal height')
						.set('display', 'toggle')
                        .inlineHelpText('Best used when border or background is added. Looks best when text content is of similar length')
                        .default(false),
					Field
						.boolean()
						.name('rounded', 'Rounded blurbs')
						.set('display', 'toggle')
						.default(true),
                    Field
                        .boolean()
                        .name('add_hover', 'Add hover effect')
						.set('display', 'toggle')
                        .inlineHelpText('Scaling hover effect is applied to all blurbs')
                        .default(false),
                    Field
                        .boolean()
                        .name('add_shadow', 'Add shadow')
						.set('display', 'toggle')
                        .default(false)
				])

	    ]
	}

}