const { Group, Field, randomId } = require('../../../build');
let GlobalGroup = require('../includes/GlobalGroup');

let Heading = require('../Heading');
let Image = require('./Image');

module.exports = class extends GlobalGroup {

	constructor() { 
		super();
		this.name("crew_members", "Crew Members")
		this.sourceId = randomId()
		this.customItemsId = randomId()
		this._build()
		return this
	}

	_build() {
		return this._buildChildrenArray(() => {
			this.makeSureOneIsTrue([
				this._showGlobalOption,
				this._showCustomOption
			])
			return this._buildChildrenArray(() => {
				(this._showCustomOption ? this.appendChildren(this.#customOptionField()) : null);
				(this._showGlobalOption ? this.#globalOptionField() : null);
				this.prependChildren(this.#sourceField());
			})
		})
	}

	// --- Show/Hide

	_showGlobalOption = true
	showGlobalOption(val = true) {
		this._showGlobalOption = !!val
		return this._build()
	}

	_showCustomOption = true
	showCustomOption(val = true) {
		this._showCustomOption = !!val
		return this._build()
	}

	// --- Settings

	_applyToCrewMembers = false
	applyToCrewMembers(fn) { 
		this._applyToCrewMembers = typeof fn =='function' ? fn : null
		return this._build()
	}
	
	_makeRepeatable = true
	makeRepeatable(val = true) {
		this._makeRepeatable = !!val
		if (!this._makeRepeatable) {
			this._minCrewMembers = 0
			this._maxCrewMembers = 1
		}
		return this._build()
	}

	_minCrewMembers = 0
	minCrewMembers(val) {
		this._minCrewMembers = parseInt(val) || 0
		if(this._minCrewMembers > 0) {this._makeRepeatable = false}
		return this._build()
	}
	
	_maxCrewMembers = 10
	maxCrewMembers(val) {
		this._maxCrewMembers = parseInt(val) || 10
		if(this._maxCrewMembers > 0) {this._makeRepeatable = false}
		return this._build()
	}

	// --- Fields

	#sourceField() {
		let typeOptions = this.getChoiceArrayFromOptions([])
		return Field
			.choice()
			.id(this.sourceId)
			.name('source', 'Crew Member Source')
			.inlineHelpText("Use the global Crew Members under 'Global Options' or a custom set.")
			.choices(typeOptions)
	}

	#globalOptionField() {
		this.appendToOptions('global', "Global")
	}

	#customOptionField() {
		let key = "custom"
		this.appendToOptions(key, "Custom Set")	
		return [
			this.itemsRepeater((group) => {
				return group.visibleIf(this.sourceId, key)
			}),
			this.#layoutGroup()
		]
	}

	itemsRepeater(applyFn = null) {
		return new Group()
			.name('crew_members', 'Crew Members')
			.id(this.customItemsId)
			.children([
				new Heading()
					.name('full_name', 'Name')
					.inlineHelpText('Name of Crew Member')
					.default('John Smith'),
				Field
					.boolean()
					.name('visible', 'Show this Item')
					.set('display', 'toggle')
					.default(true),
				new Heading()
					.name('title', 'Title')
					.inlineHelpText('Title/Role of Crew Member')
					.default('CEO'),
				Field
					.text()
					.name('description', 'Description')
					.set('allow_new_line',true),				
				new Image()
			])
			.applyIf(typeof applyFn == 'function', field => applyFn(field))
			.applyIf(!!this._applyToCrewMembers, field => this._applyToCrewMembers(field) )
			.repeat({
				min : this._minCrewMembers,
				max : this._maxCrewMembers,
				default : 2
			})
	}

	#layoutGroup() { 
		return new Group()
			.name('layout', 'Card Layout')
			.children([
				Field
					.choice()
					.name('columns', 'Columns')
					.choices([
						["3", "3 Columns"],
						["4", "4 Column"],
						["6", "6 Columns"]
					])
					.default('6'),
				Field
					.boolean()
					.name('show_image', 'Show Image')
					.inlineHelpText('Shows the image on all people displayed')
					.set('display', 'toggle')
					.default(true),
				Field
					.boolean()
					.name('show_name', 'Show Name')
					.inlineHelpText('Shows the name on all people displayed')
					.set('display', 'toggle')
					.default(true),
				Field
					.boolean()
					.name('show_title', 'Show Title/Role')
					.inlineHelpText('Shows the title/role on all people displayed')
					.set('display', 'toggle')
					.default(true),
				Field
					.boolean()
					.name('show_description', 'Show Description')
					.inlineHelpText('Shows the description on all people displayed')
					.set('display', 'toggle')
					.default(true),
				Field
					.choice()
					.name('style', 'Style')
					.choices([
						["floating", "Floating Details"],
						["stacked", "Stacked Details"]
					])
					.default('stacked'),
			])
	}

}