const { Field, config } = require('../../../build');
let GlobalGroup = require('../includes/GlobalGroup');

let Heading = require('../Heading');
let SubHeading = require('../SubHeading');
let HeadingType = require('../HeadingType');
let BasicText = require('../BasicText');
let Alignment = require('../Alignment');
let Padding = require('./Padding');
let ColumnWidth = require('../ColumnWidth');
let CssClass = require('../CssClass');

module.exports = class extends GlobalGroup {

	constructor() { 
		super();
		this.name('header', 'Header')
		this._build()
		return this
	}

	_build() {
		return this._buildChildrenArray(() => { 
			this.appendChildren(this.#headingField());
			(this._showSubHeading ? this.appendChildren(this.#subHeadingField()) : null);
			(this._showHeadingType ? this.appendChildren(this.#headingTypeField()) : null);
			(this._showText ? this.appendChildren(this.#textField()) : null);
			(this._showEnable ? this.prependChildren(this.#enableField()) : null);
			(this._showEnlargeText ? this.appendChildren(this.#enlargeTextField()) : null);
			(this._showAlignment ? this.appendChildren(this.#alignmentField()) : null);
			(this._showPadding ? this.appendChildren(this.#paddingField()) : null);
			(this._showWidth ? this.appendChildren(this.#widthField()) : null);
			(this._showCssClass ? this.appendChildren(this.#cssClassField()) : null);
		})
	}

	// --- Show/Hide

	_showEnable = true
	showEnable(val = true) {
		this._showEnable = !!val
		return this._build()
	}

	_showSubHeading = true
	showSubHeading(val = true) {
		this._showSubHeading = !!val
		return this._build()
	}

	_showHeadingType = true
	showHeadingType(val = true) {
		this._showHeadingType = !!val
		return this._build()
	}

	_showText = true
	showText(val = true) {
		this._showText = !!val
		return this._build()
	}

	_showEnlargeText = true
	showEnlargeText(val = true) {
		this._showEnlargeText = !!val
		return this._build()
	}

	_showAlignment = true
	showAlignment(val = true) {
		this._showAlignment = !!val
		return this._build()
	}

	_showPadding = true
	showPadding(val = true) {
		this._showPadding = !!val
		return this._build()
	}

	_showWidth = true
	showWidth(val = true) {
		this._showWidth = !!val
		return this._build()
	}

	_showCssClass = false
	showCssClass(val = true) {
		this._showCssClass = !!val
		return this._build()
	}

	_defaultEnable = false
	defaultEnable(val) {
		this._defaultEnable = !!val
		return this._build()
	}

	_defaultHeading = 'Module Heading'
	defaultHeading(val) {
		this._defaultHeading = (val || '').toString()
		return this._build()
	}

	_defaultSubheading = ''
	defaultSubheading(val) {
		this._defaultSubheading = (val || '').toString()
		return this._build()
	}

	_defaultText = 'Add text here'
	defaultText(val = '') {
		this._defaultText = (val || '').toString()
		return this._build()
	}

	_defaultAlignment = 'center'
	defaultAlignment(val) {
		this._defaultAlignment = (val || '').toString()
		return this._build()
	}
	
	_defaultWidth = '6'
	defaultWidth(val) {
		this._defaultWidth = (val || '').toString()
		return this._build()
	}

	// --- Fields

	#enableField() {
		return Field
			.boolean()
			.name('show', 'Show Header')
			.set("display","toggle")
			.default(!!this._defaultEnable)
	}

	#headingField() {
		return new Heading()
			.default(this._defaultHeading || 'I\'m the Heading')
	}

	#subHeadingField() { 
		return new SubHeading()
			.default(this._defaultSubheading || null)
	}

	#headingTypeField() { 
		return new HeadingType()
			.default('h2')
	}

	#textField() {
		return new BasicText()
			.name('text', 'Text')
			.default(this._defaultText)
	}

	#enlargeTextField() {
		return Field
			.boolean() 
			.name('enlarge_text', 'Enlarge Text')
			.inlineHelpText('Make the text larger.')
			.set('display', 'toggle')
			.default(false)
	}

	#alignmentField() {
		return new Alignment()
			.default(this._defaultAlignment || 'center')
	}

	#paddingField() {
		return new Padding()
			.defaultTopPadding('')
	}

	#widthField() {
		return new ColumnWidth()
			.set('choices', config('columnWidth').filter(w => parseInt(w[0]) >= 6))
			.name('width', 'Width')
			.default(this._defaultWidth)
	}

	#cssClassField() {
		return new CssClass()
			.name('class', 'Extra CSS Class')
			.inlineHelpText("Enter a CSS class for additional styling")
			.default()
	}

}