const { Field, randomId } = require('../../../build');
let GlobalGroup = require('../includes/GlobalGroup');

let Colors = require('../Colors');

module.exports = class extends GlobalGroup {

	constructor() { 
		super();
		this.name("icon", "Icon")
		this.iconTypeId = randomId()
		this._build()
		return this
	}

	_build() {
		this.makeSureOneIsTrue([
			this._showImg,
			this._showFontAwesome
		])
		return this._buildChildrenArray(() => {
			(this._showImg ? this.prependChildren(this.#imgField()) : null);
			(this._showFontAwesome ? this.appendChildren(this.#fontAwesomeField()) : null);
			this.prependChildren(this.#standardFields());
		})
	}

	// --- Show/Hide

	_showImg = true
	showImg(val = true) {
		this._showImg = !!val
		return this._build()
	}

	_showFontAwesome = true
	showFontAwesome(val = true) {
		this._showFontAwesome = !!val
		return this._build()
	}

	// --- Fields

	#standardFields() {
		let typeOptions = this.getChoiceArrayFromOptions([["none", "None"]])
		return Field
			.choice()
			.name('icon_type', 'Icon Type')
			.id(this.iconTypeId)
			.choices(typeOptions)
			.default('none')
	}

	#imgField(key = 'img') {
		this.appendToOptions(key, "Image (png/svg)")
		return Field
			.image()
			.name('image', 'Image')
			.set('resizable',false)
			.visibleIf(this.iconTypeId,key)
	}

	#fontAwesomeField(key = 'fa') {
		this.appendToOptions(key, "Font Awesome")
		return [
			Field
				.text()
				.name('fa_code', 'Font Awesome code')
				.default('fa fa-book')
				.visibleIf(this.iconTypeId, key),        
			new Colors()
				.name('fa_color', 'Font Awesome icon color')
				.visibleIf(this.iconTypeId, key)
		]
	}

}