const { Field, randomId } = require('../../../build');
let GlobalGroup = require('../includes/GlobalGroup');

module.exports = class extends GlobalGroup {

	constructor() { 
		super();
		this.name('image', 'Image')
		this.constraintTypeId = randomId()
		this._build()
		return this
	}

	_build() {
		return this._buildChildrenArray(() => {
			this.appendChildren(this.#standardFields());
			(this._showSize || this._showRatio ? this.appendChildren(this.#contraintChoiceField()) : null);
			(this._showSize ? this.appendChildren(this.#sizeField()) : null);
			(this._showRatio ? this.appendChildren(this.#ratioField()) : null);
		})
	}

	// --- Show/Hide

	_showSize = false
	showSize(val = true) {
		this._showSize = !!val
		return this._build()
	}

	_showRatio = false
	showRatio(val = true) {
		this._showRatio = !!val
		return this._build()
	}

	// --- Settings

	_defaultSize = 'cover'
	defaultSize(val) {
		this._defaultSize = val.toString()
		return this._build()
	}

	_defaultRatio = 'default'
	defaultRatio(val) {
		this._defaultRatio = val.toString()
		return this._build()
	}

	// --- Fields

	#standardFields() {
		return [
			Field
				.image()
				.name('image', 'Image')
				.inlineHelpText('Select an image to display')
				.set('resizable', false),
		]
	}

	#contraintChoiceField() { 
		return Field
			.choice()
			.name('type', 'Image Constraint Type')
			.id(this.constraintTypeId)
			.choices([
				["raw", "Show Raw Image"],
				["container","Change the Sizing"]
			])
			.default('raw')
	}

	#sizeField() {
		return Field
			.choice()
			.choices([
				["cover", "Cover the area with the Image"],
				["contain", "Contain the Image"]
			])
			.name('size', 'Image sizing')
			.visibleIf(this.constraintTypeId,'container')
			.default(this._defaultSize || 'cover')
	}

	#ratioField() {
		return Field	
			.choice()
			.choices([
				['default', 'Force Landscape'],
				['full', 'Column Height'],
				['square', 'Force Square'],
			])
			.name('ratio', 'Image Ratio')
			.visibleIf(this.constraintTypeId,'container')
			.inlineHelpText('The image will fill as much space as it can depending upon the option selected.')
			.default(this._defaultRatio || 'default')
	}

	//@todo: overlay
}