const { Field, Group, randomId } = require('../../../build');
let GlobalGroup = require('../includes/GlobalGroup');

let SwiperJsSettings = require('./SwiperJsSettings');

module.exports = class extends GlobalGroup { 
	
	constructor() {
		super()
		this.name('logos', 'Logos')
		this.sourceId = randomId()
		this.customItemsId = randomId()
		this._build()
		return this
	}

	_build() {
		this.makeSureOneIsTrue([
			this._showGlobalOption,
			this._showCustomOption
		])
		return this._buildChildrenArray(() => {
			(this._showCustomOption ? this.appendChildren(this.#customOptionField()) : null);
			this.appendChildren(this.#swiperSettingsGroup());
			(this._showGlobalOption ? this.#globalOptionField() : null);
			this.prependChildren(this.#sourceField());
		})
	}

	// --- Show/Hide

	_showEnable = true
	showEnable(val = true) {
		this._showEnable = !!val
		return this._build()
	}

	_showGlobalOption = true
	showGlobalOption(val = true) {
		this._showGlobalOption = !!val
		return this._build()
	}

	_showCustomOption = true
	showCustomOption(val = true) {
		this._showCustomOption = !!val
		return this._build()
	}
	
	// --- Settings

	_applyToLogos = false
	applyToLogos(fn) { 
		this._applyToLogos = typeof fn =='function' ? fn : null
		return this._build()
	}

	_modifySourceOptions = false
	modifySourceOptions(fn) { 
		this._modifySourceOptions = typeof fn =='function' ? fn : null
		return this._build()
	}
	
	_makeRepeatable = true
	makeRepeatable(val = true) {
		this._makeRepeatable = !!val
		if (!this._makeRepeatable) {
			this._minLogos = 0
			this._maxLogos = 1
		}
		return this._build()
	}

	_minLogos = 0
	minLogos(val) {
		this._minLogos = parseInt(val) || 0
		return this._build()
	}
	
	_maxLogos = 20
	maxLogos(val) {
		this._maxLogos = parseInt(val) || null
		return this._build()
	}

	// --- Fields


	#sourceField() {
		let typeOptions = this.getChoiceArrayFromOptions([])
		return Field
			.choice()
			.id(this.sourceId)
			.name('source', 'Logo Source')
			.inlineHelpText("Use the global logos under 'Global Options' or a custom set.")
			.choices(typeOptions)
			.applyIf(!!this._modifySourceOptions, (field) => field.choices(this._modifySourceOptions(typeOptions)))
	}

	#globalOptionField() {
		this.appendToOptions('global', "Global")
	}

	#customOptionField() {
		let key = "custom"
		this.appendToOptions(key, "Custom Set")	
		return this.itemsRepeater((group) => group.visibleIf(this.sourceId, key))
	}

	// Keep Public to access directly
	itemsRepeater(applyFn = null) {
		return new Group()
			.name('logos', 'Logos')
			.inlineHelpText('Select some logos to display')
			.id(this.customItemsId)
			.children([
				(this._showEnable ? this.#enableField() : null),
				Field
					.image()
					.name('logo', 'Logo')
					.set('resizable', false),
				Field
					.link()
					.name('link', 'Link')
			])
			.applyIf(typeof applyFn == 'function', field => applyFn(field))
			.applyIf(!!this._applyToLogos, field => this._applyToLogos(field) )
			.repeat({
				min : this._minLogos,
				max : this._maxLogos,
				default : 0
			})
	}

	#swiperSettingsGroup(){
		return new SwiperJsSettings()
	}

	#enableField() {
		return Field
			.boolean()
			.name('visible', 'Enable Logo')
			.set('display', 'toggle')
			.default(true)
	}

}