const { Field, randomId } = require('../../../build');
let GlobalGroup = require('../includes/GlobalGroup');

module.exports = class extends GlobalGroup { 
	
	constructor() {
		super()
		this.name('video', 'Video')
		this.sourceId = randomId()
		this._build()
		return this
	}

	_build() {
		return this._buildChildrenArray(() => {
			this.appendChildren(this.#standardFields());
		})
	}

	// --- Show/Hide

	_showHubspotVideo = false
	showHubspotVideo(val = true) {
		this._showHubspotVideo = !!val
		return this._build()
	}


	// --- Settings
	
	_defaultType = 'local'
	defaultType(val) {
		this._defaultType = val.toString()
		return this._build()
	}

	#standardFields() {
		(this._showHubspotVideo ? this.appendToOptions('hubspot', "HubSpot Player") : null);
		let choices = this.getChoiceArrayFromOptions([
			["local", "Local"],
			["youtube", "Youtube"],
		]);

		let defaultChoiceFound = !!choices.find(c => c[0] == this._defaultType)
		let defaultChoice = defaultChoiceFound ? this._defaultType : (choices[0][0] || '')

		return [
			Field
				.choice()
				.name('source', 'Video Source')
				.id(this.sourceId)
				.choices(choices)
				.inlineHelpText('Show an Video you have uploaded to your portal or a Youtube Video')
				.default(defaultChoice),
			
			// HS
			(this._showHubspotVideo ?
				Field
					.video()
					.name('hubspot', 'HubSpot Video')
					.inlineHelpText('Select the HubSpot Video')
					.default({ size_type : 'auto_full_width' })
					.visibleIf(this.sourceId, 'hubspot')
			: null),
		
			// YT
			Field
				.text()
				.name('youtube', 'Youtube URL')
				.inlineHelpText('Enter the URL of the youTube Video')
				.default('https://www.youtube.com/watch?v=YE7VzlLtp-4')
				.visibleIf(this.sourceId, 'youtube'),

			// Local
			Field
				.file()
				.name('video', 'Video (mp4)')
				.inlineHelpText('Select the video file to display. For best results, use <em>.mp4</em> format.')
				.set('picker', 'file')
				.default('https://commondatastorage.googleapis.com/gtv-videos-bucket/sample/BigBuckBunny.mp4')
				.visibleIf(this.sourceId, 'local'),

			// POSTER
			Field
				.image()
				.name('image', 'Poster Image')
				.inlineHelpText('Select an image to be shown on toop of the video before the play button is clicked.')
				.visibleIf(this.sourceId, 'hubspot','NOT_EQUAL')
				.set('resizable', false)
		]
	}
}