const { Group } = require('../../index');

module.exports = class extends Group {

	#childrenToAdd = []
	#optionsToAdd = []
	#oneMustBeTrue = []

	constructor() {
		super();
		this.children([])
		return this
	}

	makeSureOneIsTrue(predicates = []) {
		this.#oneMustBeTrue = Array.isArray(predicates) ? predicates : []
	}

	_buildChildrenArray(buildFn) {
		if (!!this.#oneMustBeTrue.length && this.#oneMustBeTrue.every(p => !p)) {
			this.label(`${this.data.label} (Empty)`)
			return this
		}
		if (typeof buildFn === 'function') {
			this.#childrenToAdd = [];
			this.#optionsToAdd = [];
			buildFn()
		}
		this.children((this.#childrenToAdd || []).filter(f => !!f))
		return this
	}

	appendChildren(fields = null) {
		this.#addChildren(fields)
		return this
	}

	appendChildrenIf(condition = true, fields = null) {
		if (condition) {
			return this.appendChildren(fields)
		}
		return this
	}

	prependChildren(fields = null) {
		this.#addChildren(fields, true)
		return this
	}
	
	prependChildrenIf(condition = true, fields = null) {
		if (condition) {
			return this.prependChildren(fields)
		}
		return this
	}

	#addChildren(fields = null, prepend = false) {
		if (!fields || this.data.type != 'group') { return this }	
		if (typeof fields == 'function') {
			fields = fields(this)
		}
		if (typeof fields != 'object') {
			throw new TypeError('#addChildren must return Field or Array of Fields')
		}

		fields = Array.isArray(fields) ? fields : [fields];
		this.#childrenToAdd = prepend ? fields.concat(this.#childrenToAdd) : this.#childrenToAdd.concat(fields)

		this._buildChildrenArray()
		return null
	}
	
	appendToOptions(key,label){
		if (key && label) {
			this.#optionsToAdd.push({ key, label })
			this._buildChildrenArray()
		}
		return null
	}

	getChoiceArrayFromOptions(defaultChoices) {
		let typeOptions = defaultChoices || [];
		this.#optionsToAdd.forEach(o => { typeOptions.push([o.key, o.label]) })
		return typeOptions
	}

	getChildren() {
        return this.data.children
	}

}