import { injectScripts, checkAllObjectValuesAreTrue } from "../../scriptInjector"

let hasBeenLoaded = false

/**
 * Loads Countup from CDN
 * 
 * Usage:
	import CountupLoader from 'hs-dev-helper/js-toolkit/cdnLoaders/countup/countupLoader'
	(async () => await CountupLoader.loadCountupScripts())()
 */
export const loader = class {

	countup = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadCountupScripts = ({ onload = null } = {}) => { 
		return new Promise((res, rej) => {

			const checkIfAllAreLoaded = () => {
				this.countup.ready = checkAllObjectValuesAreTrue(this.countup.loaded)
				if (this.countup.ready) {
					let payload = {Countup: window.countUp.CountUp}
					// Fire Event
					var event = new CustomEvent('CountupLoaded', { detail : payload });
					window.dispatchEvent(event);
					// OnLoad Callback
					if (this.countup.loaded.script && !!onload && typeof onload == 'function') { 
						onload(payload)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}
			
			let scripts = [
				{
					type: 'script',
					src: '//cdnjs.cloudflare.com/ajax/libs/countup.js/2.8.0/countUp.umd.min.js',
					onload: async () => {
						this.countup.loaded.script = (typeof window.countUp.CountUp == 'function')
						checkIfAllAreLoaded()
					}
				},
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}
}

export const initIfExists = async (selector) => {
	let els = document.querySelectorAll(selector);
	if (els.length) { 
		await (new loader).loadCountupScripts()

		let animationCache = [];

		function isElementInViewport (el) {
			var rect = el.getBoundingClientRect();
			return (
				rect.top >= 0 &&
				rect.left >= 0 &&
				rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) && 
				rect.right <= (window.innerWidth || document.documentElement.clientWidth)
			);
		}

		els.forEach((n) => {
			if(isElementInViewport(n) && animationCache.indexOf(n) < 0) {
				let { CountUp } = window.countUp
				var countUp = new CountUp(n, n.innerHTML);
				n.closest('.countup-cloak').classList.remove('countup-cloak--cloaked');
				countUp.start();
				animationCache.push(n);
			}
		});
	}
}

export default new loader