import { injectScripts, checkAllObjectValuesAreTrue } from "../../scriptInjector"

let hasBeenLoaded = false

/**
 * Loads jQuery Library from CDN
 * 
 * Usage:
	import jQueryLoader from 'hs-dev-helper/js-toolkit/cdnLoaders/jQuery/jQueryLoader'
	(async () => await jQueryLoader.loadjQueryScripts())()
 */
export const loader = class {

	jQuery = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadjQueryScripts = (onload = null) => { 
		return new Promise((res, rej) => { 
			const checkIfAllAreLoaded = () => {
				this.jQuery.ready = checkAllObjectValuesAreTrue(this.jQuery.loaded)
				if (this.jQuery.ready) {
					let payload = { jQuery: window.jQuery }
					// Fire Event
					var event = new CustomEvent('jQueryLoaded', { detail : payload });
					window.dispatchEvent(event);
					// Onload Callback
					if (this.jQuery.loaded.script && !!onload && typeof onload == 'function') { 
						onload(window.jQuery)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}	
			
			let scripts = [
				{
					type: 'script',
					src: 'https://code.jquery.com/jquery-3.6.3.min.js',
					onload: () => {
						this.jQuery.loaded.script = (typeof window.jQuery == 'function')
						checkIfAllAreLoaded()
					}
				}
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}

}

export default new loader