import { injectScripts, checkAllObjectValuesAreTrue } from "hs-dev-helper/js-toolkit/scriptInjector"

let hasBeenLoaded = false

/**
 * Loads Luxon Library from CDN
 * 
 * Usage:
	import luxonLoader from '...cdnLoaders/luxon/luxonLoader'
	(async () => await luxonLoader.loadLuxonScripts())()
 */
export const loader = class {

	luxon = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadLuxonScripts = (onload = null) => { 
		return new Promise((res, rej) => { 
			const checkIfAllAreLoaded = () => {
				this.luxon.ready = checkAllObjectValuesAreTrue(this.luxon.loaded)
				if (this.luxon.ready) {
					let payload = { luxon: window.luxon }
					// Fire Event
					var event = new CustomEvent('LuxonLoaded', { detail : payload });
					window.dispatchEvent(event);
					// Onload Callback
					if (this.luxon.loaded.script && !!onload && typeof onload == 'function') { 
						onload(window.luxon)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}	
			
			let scripts = [
				{
					type: 'script',
					src: 'https://cdn.jsdelivr.net/npm/luxon@3.3.0/build/global/luxon.min.js',
					onload: () => {
						this.luxon.loaded.script = (typeof window.luxon == 'object')
						checkIfAllAreLoaded()
					}
				}
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}

}

export default new loader