export default class {

	constructor(sweetAlertInstance) {
		// Set SweetAllert object
		this.Swal = sweetAlertInstance
		// Fire Event
		var event = new CustomEvent('sweetAlertHelper',this);
		window.dispatchEvent(event);
		// Reset
		this.reset();
		return this
	}
    
    reset = () => {
        this.data = {
            showConfirmButton: true,
            showCancelButton: false,
            showDenyButton: false,
            confirmButtonColor: '#61CA85',
            cancelButtonColor: '#0A0030',
            denyButtonColor: '#FF434D',
            timer: 3000,
            timerProgressBar: true,
            didOpen: (toast) => {
                toast.addEventListener('mouseenter', this.Swal.stopTimer)
                toast.addEventListener('mouseleave', this.Swal.resumeTimer)
            }
        }
    }

    fire = async (args = {}) => {
        let data = this.data
        this.reset();
        return await this.Swal.mixin(data).fire(args)
    }

    /*
    |--------------------------------------------------------------------------
    | General
    |--------------------------------------------------------------------------
    */

    mixin = (args = {}) => {
        this.data = Object.assign({},this.data,args)
        return this
    }

    /*
    |--------------------------------------------------------------------------
    | Types
    |--------------------------------------------------------------------------
    */
	
	noTimer = () => {
        this.data = Object.assign({}, this.data, {
            timer: false,
        })
        return this
    }

	noToast = () => {
        this.data = Object.assign({}, this.data, {
            toast: false,
        })
        return this
    }
    
    isToast = () => {
        this.data = Object.assign({}, this.data, {
            toast: true,
            position: 'bottom-end',
            showConfirmButton: false,
            timer: 3000,
            text: '',
            html : ''
        })
        return this
    }

    isInfo = () => {
        this.data = Object.assign({},this.data,{icon: 'info'})
        return this
    }
    
    isError = () => {
        this.data = Object.assign({},this.data,{icon: 'error'})
        return this
    }
    
    isWarning = () => {
        this.data = Object.assign({},this.data,{icon: 'warning'})
        return this
    }
    
    isSuccess = () => {
        this.data = Object.assign({},this.data,{icon: 'success'})
        return this
    }

    /*
    |--------------------------------------------------------------------------
    | Actions
    |--------------------------------------------------------------------------
    */

    success = () => {
        return this.isSuccess().mixin({
            title: 'Success!',
        });
    }

    error = (errors) => {
        return this.isError().mixin({
            title: 'Uh-Oh! There was an error',
            html: this.constructor.flattenErrorResponsetoString(errors),
        });
    }
    
    confirm = () => {
        return this.isWarning().mixin({
			text: "Are you sure?",
			showCancelButton: true,
            confirmButtonText: 'Yes!',
            cancelButtonText: 'Cancel'
        });
	}
	
	confirmAction = (confirmText) => {
        return this.confirm().mixin({
			text: "Are you sure?",
            confirmButtonText: confirmText || 'Yes!',
        });
    }
    
    confirmDelete = (victim) => {
        this.confirm().mixin({
			text: "Are you sure?",
			confirmButtonText: 'Yes, Delete!',
            confirmButtonColor: '#d33',
            cancelButtonColor: '#61CA85',
        });
        if (victim) {
            this.data['title'] = 'Delete "' + victim + '"?'
        }
        return this
    }
    
    created = (target) => {
        return this.isSuccess().mixin({
            title: 'Created!',
            text: target ? '"'+target +'" was created!' : 'The resource has been created.',
        });
    }
    
    saved = (target) => {
        return this.isSuccess().mixin({
            title: 'Saved!',
            text: target ? '"'+target +'" was saved!' : 'The resource has been saved.',
        });
    }
    
    updated = (target) => {
        return this.isSuccess().mixin({
            title: 'Updated!',
            text: target ? '"'+target +'" was updated!' : 'The resource has been updated.',
        });
    }

    deleted = (victim) => {
        this.isSuccess().mixin({
            title: 'Deleted!',
            text: victim ? ''+victim +' was deleted!' : 'The resource has been deleted.',
        });
        return this
    }
    
    duplicated = (target) => {
        return this.isSuccess().mixin({
            title: 'Duplicated!',
            text: target ? '"'+target +'" was duplicated!' : 'The resource has been duplicated.',
        });
	}
	
	loading = () => {
        return this.isInfo().mixin({
			title: 'Please Wait... Processing',
            timer: false,
			allowOutsideClick: false,
			didOpen: () => {this.Swal.showLoading()},
        });
	}
	
	close = () => {
        this.Swal.close()
	}
	
	static flattenErrorResponsetoString(error) {
		if (!error || error.response == undefined) { return error || 'An Unknown Error Occurred.';}
		return error.response.data.errors ?
			_.flatten(_.toArray(error.response.data.errors)).join('<br>') :
			error.response.data.message;
	}

}