import { injectScripts, checkAllObjectValuesAreTrue } from "../../scriptInjector"
import sweetAlert from './sweetAlertHelper'

let hasBeenLoaded = false

/**
 * Loads SweetAlert Library from CDN
 * 
 * Usage:
	import sweetAlertLoader from 'hs-dev-helper/js-toolkit/cdnLoaders/sweetAlert/sweetAlertLoader'
	(async () => await sweetAlertLoader.loadSweetAlertScripts())()
 */
export const loader = class {

	sweetAlert = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadSweetAlertScripts = (onload = null) => { 
		return new Promise((res, rej) => { 
			const checkIfAllAreLoaded = () => {
				this.sweetAlert.ready = checkAllObjectValuesAreTrue(this.sweetAlert.loaded)
				if (this.sweetAlert.ready) {
					let payload = {
						swal: window.sweetAlert,
						helper : new sweetAlert(window.sweetAlert)
					}
					// Fire Event
					var event = new CustomEvent('SweetAlertLoaded', { detail : payload });
					window.dispatchEvent(event);
					// Add to Window Util
					window._util.swal = payload.helper
					// Onload Callback
					if (this.sweetAlert.loaded.script && !!onload && typeof onload == 'function') { 
						onload(window.sweetAlert)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}	
			
			let scripts = [
				{
					type: 'script',
					src: 'https://cdn.jsdelivr.net/npm/sweetalert2@11.5.2/dist/sweetalert2.all.min.js',
					onload: () => {
						this.sweetAlert.loaded.script = (typeof window.sweetAlert == 'function')
						checkIfAllAreLoaded()
					}
				}
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}


}


/**
 * Load SweetAlert from Vue
 * 
 * Usage
	import { VueSweetAlertMixin } from 'hs-dev-helper/js-toolkit/cdnLoaders/sweetAlert/sweetAlertLoader';
	...
	mixins: [VueSweetAlertMixin],
	...
	computed: {
		ready() { 
			return !!this.sweetAlert.ready
		},
		...
	}
*/
export const VueSweetAlertMixin = {
	created() {
		window.addEventListener('SweetAlertLoaded', ({ detail }) => {
			let { helper } = detail
			this.sweetAlert.ready = true
			this.swal = helper
		});
	},
	beforeMount() {
		(async () => await (new loader).loadSweetAlertScripts())()
	},
	data: () => ({
		swal : null,
		sweetAlert : {ready: false}
	}),
}

export default new loader