import { injectScripts, checkAllObjectValuesAreTrue } from "../../scriptInjector"

let hasBeenLoaded = false

/**
 * Loads SwiperJs from CDN
 * 
 * Usage:
	import SwiperJsLoader from 'hs-dev-helper/js-toolkit/cdnLoaders/swiperJs/swiperJsLoader'
	(async () => await SwiperJsLoader.loadSwiperJsScripts())()
 */
export const loader = class {

	swiperJs = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadSwiperJsScripts = ({ onload = null } = {}) => { 
		return new Promise((res, rej) => {

			const checkIfAllAreLoaded = () => {
				this.swiperJs.ready = checkAllObjectValuesAreTrue(this.swiperJs.loaded)
				if (this.swiperJs.ready) {
					let payload = {SwiperJs: window.Swiper}
					// Fire Event
					var event = new CustomEvent('SwiperJsLoaded', { detail : payload });
					window.dispatchEvent(event);
					// OnLoad Callback
					if (this.swiperJs.loaded.script && !!onload && typeof onload == 'function') { 
						onload(payload)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}
			
			let scripts = [
				{
					type: 'script',
					src: '//unpkg.com/swiper@7/swiper-bundle.min.js',
					onload: async () => {
						this.swiperJs.loaded.script = (typeof Swiper == 'function')
						checkIfAllAreLoaded()
					}
				},
				{
					type: 'style',
					src: '//unpkg.com/swiper@7/swiper-bundle.min.css',
					onload: async () => {
						this.swiperJs.loaded.script = true
						checkIfAllAreLoaded()
					}
				}
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}

}

export const initIfExists = async (selector, options) => {
	let containers = document.querySelectorAll(selector);
	if (containers.length) { 
		await (new loader).loadSwiperJsScripts()

		options = Object.assign({}, {
			slidesPerView: 1,
			loop: false,
			grabCursor: true,
			speed: 500,
			pagination: {
				el: '.swiper-pagination',
				type: 'bullets',
				clickable: true,
			},
			navigation: {
				nextEl: '.swiper-button-next',
				prevEl: '.swiper-button-prev',
			},
			autoplay: {
			  delay: 2500,
			  disableOnInteraction: true,
			},
		}, options);

		containers.forEach((el)=>{
			let swiper = el.querySelector('.swiper-container');

			let swiperAttr = swiper.hasAttribute('data-swiper') ? swiper.getAttribute('data-swiper') : "{}"
			let elOptions = Object.assign({}, options, JSON.parse(swiperAttr) || {})

			new Swiper(swiper,elOptions);
		});

	}

}

export default new loader