import { injectScripts, checkAllObjectValuesAreTrue } from "../../scriptInjector"

let hasBeenLoaded = false

/**
 * Loads VideoJs from CDN
 * 
 * Usage:
	import VideoJsLoader from 'hs-dev-helper/js-toolkit/cdnLoaders/videoJs/videoJsLoader'
	(async () => await VideoJsLoader.loadVideoJsScripts())()
 */
export const loader = class {

	videoJs = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadVideoJsScripts = ({ onload = null } = {}) => { 
		return new Promise((res, rej) => {

			const checkIfAllAreLoaded = () => {
				this.videoJs.ready = checkAllObjectValuesAreTrue(this.videoJs.loaded)
				if (this.videoJs.ready) {
					let payload = {VideoJs: window.videojs}
					// Fire Event
					var event = new CustomEvent('VideoJsLoaded', { detail : payload });
					window.dispatchEvent(event);
					// OnLoad Callback
					if (this.videoJs.loaded.script && !!onload && typeof onload == 'function') { 
						onload(payload)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}
			
			let scripts = [
				{
					type: 'script',
					src: '//vjs.zencdn.net/8.3.0/video.min.js',
					onload: async () => {
						injectScripts([{
							type: 'script',
							src: '//cdnjs.cloudflare.com/ajax/libs/videojs-youtube/3.0.1/Youtube.min.js',
							onload: () => {
								this.videoJs.loaded.script = (typeof videojs == 'function')
								checkIfAllAreLoaded()
							}
						}])
					}
				},
				{
					type: 'style',
					src: '//vjs.zencdn.net/8.3.0/video-js.css',
					onload: () => {
						this.videoJs.loaded.style = true
						checkIfAllAreLoaded()
					}
				}
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}
}

export const initIfExists = async (selector, options) => {
	let containers = document.querySelectorAll(selector);
	if (containers.length) { 
		await (new loader).loadVideoJsScripts()

		containers.forEach((el)=>{
			let vjs = el.querySelector('.video-js');
			let url = vjs.getAttribute('data-url');
			let type = vjs.getAttribute('data-type');

			videojs(vjs, {
				fluid: true,
				fill: true,
				responsive: true,
				sources: [{
				  src: url,
				  type: type
				}]
			});
		});

	}

}

export default new loader