import { injectScripts, checkAllObjectValuesAreTrue } from "../../scriptInjector"
import vueRouterLoader from './vueRouterLoader'

let hasBeenLoaded = false

/**
 * Loads Vue from CDN
 * 
 * Usage:
	import VueLoader from 'hs-dev-helper/js-toolkit/cdnLoaders/vue/vueLoader'
	(async () => await VueLoader.loadVueScripts({ vueVersion: 3 }))()
 */
export const loader = class {

	vue = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadVueScripts = ({ vueVersion = 2, includeRouter = false, onload = null } = {}) => { 
		return new Promise((res, rej) => {

			const checkIfAllAreLoaded = () => {
				this.vue.ready = checkAllObjectValuesAreTrue(this.vue.loaded)
				if (this.vue.ready) {
					let payload = {Vue: window.Vue}
					// Fire Event
					var event = new CustomEvent('VueLoaded', { detail : payload });
					window.dispatchEvent(event);
					// OnLoad Callback
					if (this.vue.loaded.script && !!onload && typeof onload == 'function') { 
						onload(payload)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}

			let vueCdnScript;
			switch (vueVersion) {
				case 2:
					//https://v2.vuejs.org/v2/guide/installation.html
					vueCdnScript = (IS_DEV || false) ?
						'https://cdn.jsdelivr.net/npm/vue@2.7.13/dist/vue.js' :
						'https://cdn.jsdelivr.net/npm/vue@2.7.13/dist/vue.min.js';
					break;
				case 3:
					//https://v2.vuejs.org/v2/guide/installation.html
					vueCdnScript = (IS_DEV || false) ?
						'https://unpkg.com/vue@3/dist/vue.global.js' :
						'https://unpkg.com/vue@3/dist/vue.global.prod.js';
					break;
			}

			let checkExists = () => { 
				let returnType;
				switch (vueVersion) {
					case 2:
						returnType = 'function'
						break;
					case 3:
						returnType = 'object'
						break;
				}
				return (!!window.Vue && typeof window.Vue == returnType)
			}
			
			let scripts = [
				{
					type: 'script',
					src: vueCdnScript,
					onload: async () => {
						this.vue.loaded.script = checkExists()
						if (includeRouter) {
							await vueRouterLoader.loadVueRouterScripts({ vueVersion })
						}
						checkIfAllAreLoaded()
					}
				}
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}


}

export default new loader