import { injectScripts, checkAllObjectValuesAreTrue } from "../../scriptInjector"

let hasBeenLoaded = false

/**
 * Loads Vue from CDN
 * 
 * Usage:
	import VueRouterLoader from 'hs-dev-helper/js-toolkit/cdnLoaders/vue/vueRouterLoader'
	(async () => await VueRouterLoader.loadVueRouterScripts({ vueVersion: 3 }))()
 */
export const loader = class {

	vueRouter = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadVueRouterScripts = ({ vueVersion = 2, onload = null } = {}) => { 
		return new Promise((res, rej) => {

			const checkIfAllAreLoaded = () => {
				this.vueRouter.ready = checkAllObjectValuesAreTrue(this.vueRouter.loaded)
				if (this.vueRouter.ready) {
					let payload = {VueRouter: window.VueRouter}
					// Fire Event
					var event = new CustomEvent('VueRouterLoaded', { detail : payload });
					window.dispatchEvent(event);
					// OnLoad Callback
					if (this.vueRouter.loaded.script && !!onload && typeof onload == 'function') { 
						onload(payload)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}

			let vueRouterCdnScript;
			switch (vueVersion) {
				case 2:
					//https://v3.router.vuejs.org/installation.html#direct-download-cdn
					vueRouterCdnScript = (IS_DEV || false) ?
						'https://unpkg.com/vue-router@3/dist/vue-router.js' :
						'https://unpkg.com/vue-router@3/dist/vue-router.min.js';
					break;
				case 3:
					//https://router.vuejs.org/installation.html#direct-download-cdn
					vueRouterCdnScript = (IS_DEV || false) ?
						'https://unpkg.com/vue-router@4/dist/vue-router.global.js' :
						'https://unpkg.com/vue-router@4/dist/vue-router.global.prod.js';
					break;
			}

			let checkExists = () => { 
				let returnType;
				switch (vueVersion) {
					case 2:
						returnType = 'function'
						break;
					case 3:
						returnType = 'object'
						break;
				}
				return (!!window.Vue && typeof window.Vue == returnType) && (!!window.VueRouter && typeof window.VueRouter == returnType)
			}
			
			let scripts = [
				{
					type: 'script',
					src: vueRouterCdnScript,
					onload: () => {
						this.vueRouter.loaded.script = checkExists()
						checkIfAllAreLoaded()
					}
				}
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}


}

export default new loader