import { injectScripts, checkAllObjectValuesAreTrue } from "hs-dev-helper/js-toolkit/scriptInjector"

let hasBeenLoaded = false

/**
 * Loads VueI18n Library from CDN
 * 
 * Usage:
	import vueI18nLoader from '...cdnLoaders/vueI18n/vueI18nLoader'
	(async () => await vueI18nLoader.loadVueI18nScripts())()
 */
export const loader = class {

	vueI18n = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadVueI18nScripts = (onload = null) => { 
		return new Promise((res, rej) => { 
			const checkIfAllAreLoaded = () => {
				this.vueI18n.ready = checkAllObjectValuesAreTrue(this.vueI18n.loaded)
				if (this.vueI18n.ready) {
					let payload = { vueI18n: window.VueI18n }
					// Fire Event
					var event = new CustomEvent('VueI18nLoaded', { detail : payload });
					window.dispatchEvent(event);
					// Onload Callback
					if (this.vueI18n.loaded.script && !!onload && typeof onload == 'function') { 
						onload(window.vueI18n)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}	
			
			let scripts = [
				{
					type: 'script',
					src: 'https://unpkg.com/vue-i18n@8',
					onload: () => {
						this.vueI18n.loaded.script = (typeof window.VueI18n == 'function')
						checkIfAllAreLoaded()
					}
				}
			]
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}

}

export default new loader