import { slugify } from 'hs-dev-helper/js-toolkit/util/str';

/**
 * Will find the page heading within a target
 * If a destination is passed, will create scroll links to the sections
 * 
 * Usage:
	CreateNavFromHeadings.initIfExists({
		parent : '.module-outer.plain-text-module',
		source: '.hs_cos_wrapper_type_rich_text',
		navDest : '.page-navigation'
	});
 */

export default (function () { 
	return {
		debug: false,
		
		parent: null,
		source: null,
		navDest: null,
		
		init: function ({ parent, source, navDest, selector = 'h2' } = {}) {
			this.log(`Initializing if ${parent} exists...`);
			this.parent = document.querySelector(parent) || null
			// Check
			if (!this.parent) {
				this.log(`Container with selector ${parent} not found!`);
				return;
			}
			// Exists
			this.log(`Container exists...`);
			
			// Check for Source and Dest
			this.source = document.querySelector(source) || null
			if (!this.source) {
				this.log(`Source not found`);
				return;
			}
			// Find Headings
			let pageHeadingsFound = this.findPageHeadings(selector)
			this.log(`Found Page headings`, pageHeadingsFound);
			
			// Create Nav
			this.navDest = document.querySelector(navDest) || null
			if (!this.navDest) {
				this.log(`Nav Dest not found`);
				return pageHeadingsFound;
			}
			this.createNav(pageHeadingsFound)
			this.log(`Created Nav`);

			// Return
			return pageHeadingsFound;
		},
		
		// Get Headings
		findPageHeadings: function (selector = 'h2') { 
			let sectionHeadings = []
			let headingsFound = this.source.querySelectorAll(selector);
			Array.from(headingsFound).forEach((h,i) => {
				let id = slugify(h.textContent+'-'+i)
				sectionHeadings.push({
					el : h,
					id,
					label : h.textContent
				});
			})
			return sectionHeadings || [];
		},
		
		// Create Nav from Headings Found
		createNav: function (headings = []) { 
			headings.forEach(h => {
				h.el.setAttribute('id', h.id);
				let li = document.createElement('li')
				li.innerHTML = `<a class="nav-link" href="#${h.id}">${h.label}</a>`;
				this.navDest.appendChild(li)
			})
		},

		initIfExists: function ({ parent, source, navDest } = {}) {
			this.init({ parent, source, navDest })
        },
		

		log: function(msg){
            if (this.debug) {
                console.log(msg);
            } 
		}
	}
})();